uniform float Time;
float4x4 mWorldViewProj;
float WaveHeight;
float WaveLength;
float WaveSpeed;
float4 AddedColor;
float4 MultiColor;
float UnderWater;
float WaveDisplacement;
float WaveRepetition;
float RefractionFactor;

struct VS_OUTPUT
{
    float4 Position : POSITION;
    float4 TexCoord : TEXCOORD0;
    float  Addition : TEXCOORD1;
};

VS_OUTPUT vertexMain( float4 vPosition : POSITION,
                      float2 texCoord : TEXCOORD0 )
{
    VS_OUTPUT Output;
    Output.Position = mul( vPosition, mWorldViewProj );
   Output.Addition = ( sin( ( vPosition.x / WaveLength ) + ( Time * WaveSpeed / 10000.0 ) ) ) +
                 ( cos( ( vPosition.z / WaveLength ) + ( Time * WaveSpeed / 10000.0 ) ) );
   Output.TexCoord = Output.Position;
   Output.TexCoord.y += Output.Addition * WaveHeight;
    return Output;
}

struct PS_OUTPUT
{
    float4 RGBColor : COLOR0;
};

texture ReflectionTexture;
sampler MySampler = sampler_state
{
    Texture = ReflectionTexture;
    AddressU = CLAMP;
    AddressV = CLAMP;
};

PS_OUTPUT pixelMain( VS_OUTPUT In )
{
   PS_OUTPUT Output;
   
   float4 projCoord = In.TexCoord / In.TexCoord.w;
   projCoord += float4( 1.0, 1.0, 1.0, 1.0 );
   projCoord *= 0.5;
   projCoord.x += sin( In.Addition * WaveRepetition ) * ( WaveDisplacement / 1000.0 );
   projCoord.y += cos( In.Addition * WaveRepetition ) * ( WaveDisplacement / 1000.0 );
   projCoord = clamp( projCoord, 0.001, 0.999 );

   if( UnderWater == 1.0 )
      projCoord.y = 1.0 - projCoord.y;

   float4 refTex = tex2D( MySampler, projCoord );
   refTex = (refTex + AddedColor) * MultiColor;
   Output.RGBColor = refTex;

   if( UnderWater == 1.0 )
      Output.RGBColor *= (MultiColor / 1.1);

   Output.RGBColor.a = RefractionFactor;
   return Output;
} 